@@ -1,9 +1,11 @@
 """
-UBX Protocol Polling payload definitions
+UBX Protocol POLL payload definitions.
 
 THESE ARE THE PAYLOAD DEFINITIONS FOR _POLL_ MESSAGES _TO_ THE RECEIVER
-(e.g. query configuration; request monitoring, receiver management, logging or sensor fusion status)
-Response payloads are defined in UBX_PAYLOADS_GET
+(e.g. query configuration; request monitoring, receiver management,
+logging or sensor fusion status).
+
+Response payloads are defined in UBX_PAYLOADS_GET.
 
 NB: Attribute names must be unique within each message class/id
 
@@ -11,7 +13,7 @@ Created on 27 Sep 2020
 
 Information sourced from public domain u-blox Interface Specifications © 2013-2021, u-blox AG
 
-:author: semuadmin
+:author: semuadmin (Steve Smith)
 """
 
 from pyubx2.ubxtypes_core import U1, U2, U4
@@ -210,6 +212,7 @@ UBX_PAYLOADS_POLL = {
     "RXM-SVSI": {},
     "RXM-TM": {},
     # *************************************************
+    "SEC-OSNMA": {},
     "SEC-SIG": {},
     "SEC-SIGLOG": {},
     "SEC-SIGN": {},
