@@ -7,9 +7,13 @@ Created on 3 Oct 2020
 
 @author: semuadmin
 """
+
 # pylint: disable=line-too-long, invalid-name, missing-docstring, no-member
 
+import os
+from io import BytesIO
 import unittest
+from logging import ERROR
 
 from pyubx2 import (
     UBXMessage,
@@ -22,7 +26,10 @@ from pyubx2 import (
     SET,
     POLL,
     VALCKSUM,
+    ERR_LOG,
     ERR_RAISE,
+    SET_LAYER_FLASH,
+    TXN_NONE,
 )
 from pyubx2.ubxhelpers import (
     cfgkey2name,
@@ -31,6 +38,8 @@ from pyubx2.ubxhelpers import (
     bytes2val,
 )
 
+DIRNAME = os.path.dirname(__file__)
+
 
 class ExceptionTest(unittest.TestCase):
     def setUp(self):
@@ -44,9 +53,9 @@ class ExceptionTest(unittest.TestCase):
         pass
 
     def testInvMode(self):  # test invalid mode
-        EXPECTED_ERROR = "Invalid msgmode 3 - must be 0, 1 or 2"
+        EXPECTED_ERROR = "Invalid msgmode 4 - must be 0, 1 or 2"
         with self.assertRaisesRegex(UBXMessageError, EXPECTED_ERROR):
-            UBXMessage("CFG", "CFG-MSG", 3, msgClass=240, msgID=5)
+            UBXMessage("CFG", "CFG-MSG", 4, msgClass=240, msgID=5)
 
     def testAckCkT(self):  # bad checksum
         EXPECTED_ERROR = "Message checksum (.*) invalid - should be (.*)"
@@ -320,21 +329,22 @@ class ExceptionTest(unittest.TestCase):
             UBXMessage("CFG", "CFG-VALGET", GET, version=0, layer=0)
 
     def testParseMode(self):  # test invalid parse message mode
-        EXPECTED_ERROR = "Invalid message mode 3 - must be 0, 1 or 2"
+        EXPECTED_ERROR = "Invalid message mode 4 - must be 0, 1, 2 or 3"
         with self.assertRaisesRegex(UBXParseError, EXPECTED_ERROR):
             UBXReader.parse(
-                b"\xb5b\x05\x01\x02\x00\x06\x01\x0f\x38", validate=VALCKSUM, msgmode=3
+                b"\xb5b\x05\x01\x02\x00\x06\x01\x0f\x38", validate=VALCKSUM, msgmode=4
             )
 
     def testParseMode2(self):  # test parser with incorrect mode for input message
-        EXPECTED_ERROR = "Unknown message type clsid (.*), msgid (.*), mode GET"
-        with self.assertRaisesRegex(UBXParseError, EXPECTED_ERROR):
-            UBXReader.parse(self.mga_ini, validate=VALCKSUM, quitonerror=ERR_RAISE)
+        EXPECTED_ERROR = "Unknown message type b'\\x13\\x40', mode GET. Check 'msgmode' setting is appropriate for data stream"
+        with self.assertRaises(UBXMessageError) as context:
+            UBXReader.parse(self.mga_ini, validate=VALCKSUM)
+        self.assertTrue(EXPECTED_ERROR in str(context.exception))
 
     def testStreamMode(self):  # test invalid stream message mode
-        EXPECTED_ERROR = "Invalid stream mode 3 - must be 0, 1 or 2"
+        EXPECTED_ERROR = "Invalid stream mode 4 - must be 0, 1, 2 or 3"
         with self.assertRaisesRegex(UBXStreamError, EXPECTED_ERROR):
-            UBXReader(None, validate=VALCKSUM, msgmode=3)
+            UBXReader(None, validate=VALCKSUM, msgmode=4)
 
     def testVal2Bytes(self):  # test invalid attribute type
         EXPECTED_ERROR = "Unknown attribute type Z001"
@@ -347,22 +357,76 @@ class ExceptionTest(unittest.TestCase):
             bytes2val(b"\x01", "Z001")
 
     def testWRONGMSGMODE(self):  # test parse of SET message with GET msgmode
-        EXPECTED_ERROR = (
-            "Unknown message type clsid (.*), msgid (.*), mode GET\n"
-            + "Check 'msgmode' keyword argument is appropriate for data stream"
-        )
+        EXPECTED_ERROR = "Unknown message type b'\\x06\\x8a', mode GET. Check 'msgmode' setting is appropriate for data stream"
         EXPECTED_RESULT = "<UBX(CFG-VALSET, version=0, ram=1, bbr=1, flash=0, action=0, reserved0=0, CFG_UART1_BAUDRATE=9600)>"
-        res = UBXMessage(
-            "CFG",
-            "CFG-VALSET",
-            SET,
-            payload=b"\x00\x03\x00\x00\x01\x00\x52\x40\x80\x25\x00\x00",
-        )
-        with self.assertRaisesRegex(UBXParseError, EXPECTED_ERROR):
-            msg = UBXReader.parse(res.serialize(), quitonerror=ERR_RAISE)
+        with self.assertRaises(UBXMessageError) as context:
+            res = UBXMessage(
+                "CFG",
+                "CFG-VALSET",
+                SET,
+                payload=b"\x00\x03\x00\x00\x01\x00\x52\x40\x80\x25\x00\x00",
+            )
+            msg = UBXReader.parse(res.serialize())
+        self.assertTrue(EXPECTED_ERROR in str(context.exception))
         msg = UBXReader.parse(res.serialize(), msgmode=SET)
         self.assertEqual(str(msg), EXPECTED_RESULT)
 
+    def testprematurestreamend(self):  # test truncated UBX stream
+
+        class SlowReader(BytesIO):
+            def read(self, size: int) -> bytes:
+                b = super().read(min(size, 16))
+                return b
+
+        ubx = b"\xb5b\x06\x8b4\x00\x01\x07\x00\x00\x05\x00R\x10\x01\x06\x00R\x10\x00\x07\x00R\x10\x00\x02\x00R \x01\x03\x00R \x00\x04\x00R \x00\x08\x00R \x00\t\x00R \x01\x01\x00R@\x00\x96\x00\x00\x85\x01"
+
+        with self.assertRaisesRegex(
+            UBXStreamError,
+            "Serial stream terminated unexpectedly. 54 bytes requested, 16 bytes returned.",
+        ):
+            for _, parsed_data in UBXReader(SlowReader(ubx), quitonerror=ERR_RAISE):
+                print(parsed_data)
+
+    def testNMEABADEND(self):  # test truncated NMEA file
+        i = 0
+        with self.assertRaisesRegex(
+            UBXStreamError,
+            "Serial stream terminated unexpectedly. Line requested, 30 bytes returned.",
+        ):
+            with open(
+                os.path.join(DIRNAME, "pygpsdata-NMEABADEND.log"), "rb"
+            ) as stream:
+                ubr = UBXReader(stream, quitonerror=ERR_RAISE)
+                for raw, parsed in ubr:
+                    # print(f'"{parsed}",')
+                    i += 1
+
+    def testBADConfigSet(self):  # test invalid configuration database value type
+        with self.assertRaises(TypeError):
+            UBXMessage.config_set(
+                layers=SET_LAYER_FLASH,
+                transaction=TXN_NONE,
+                cfgData=[(0x20920006, 0)],
+            )
+        with self.assertRaises(TypeError):
+            UBXMessage.config_set(
+                layers=SET_LAYER_FLASH,
+                transaction=TXN_NONE,
+                cfgData=[(0x209100E0, b"\x00")],
+            )
+        with self.assertRaises(TypeError):
+            UBXMessage.config_set(
+                layers=SET_LAYER_FLASH,
+                transaction=TXN_NONE,
+                cfgData=[(0x5005002A, b"\x00")],
+            )
+        with self.assertRaises(TypeError):
+            UBXMessage.config_set(
+                layers=SET_LAYER_FLASH,
+                transaction=TXN_NONE,
+                cfgData=[(0x40110069, 0.1), (0x40110068, "0")],
+            )
+
 
 if __name__ == "__main__":
     # import sys;sys.argv = ['', 'Test.testName']
